<?php

namespace FiloBlu\Rma\Plugin\Rma\Api\RmaRepositoryInterface;


use Magento\Framework\App\ResourceConnection;
use Magento\Sales\Api\OrderRepositoryInterface;
use Psr\Log\LoggerInterface;
use FiloBlu\Rma\Api\RmaExtensionAttributesRepositoryInterface;
use Magento\Rma\Model\ResourceModel\Item\CollectionFactory as ItemCollectionFactory;
use FiloBlu\Rma\Helper\RmaHelper;



class AfterGet
{
    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;

    /**
     * @var \Magento\Rma\Api\Data\ItemExtensionFactory
     */
    private $extensionInterfaceFactory;
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;
    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var RmaExtensionAttributesRepositoryInterface
     */
    protected $rmaExtensionAttributesRepository;

    /**
     * @var ItemCollectionFactory
     */
    protected $itemCollectionFactory;
    /**
     * @var RmaHelper
     */
    private $rmaHelper;


    /**
     * AfterGet constructor.
     * @param OrderRepositoryInterface $orderRepository
     * @param \Magento\Rma\Api\Data\ItemExtensionFactory $extensionInterfaceFactory
     * @param ResourceConnection $resourceConnection
     * @param LoggerInterface $logger
     * @param RmaExtensionAttributesRepositoryInterface $rmaExtensionAttributesRepository
     * @param ItemCollectionFactory $itemCollectionFactory
     * @param RmaHelper $rmaHelper
     */
    public function __construct(
        OrderRepositoryInterface $orderRepository,
        \Magento\Rma\Api\Data\ItemExtensionFactory $extensionInterfaceFactory,
        ResourceConnection $resourceConnection,
        LoggerInterface $logger,
        RmaExtensionAttributesRepositoryInterface $rmaExtensionAttributesRepository,
        ItemCollectionFactory $itemCollectionFactory,
        RmaHelper $rmaHelper)

    {
        $this->orderRepository = $orderRepository;
        $this->extensionInterfaceFactory = $extensionInterfaceFactory;
        $this->resourceConnection = $resourceConnection;
        $this->logger = $logger;
        $this->rmaExtensionAttributesRepository = $rmaExtensionAttributesRepository;
        $this->itemCollectionFactory = $itemCollectionFactory;
        $this->rmaHelper = $rmaHelper;
    }

    /**
     * @param \Magento\Rma\Api\RmaRepositoryInterface $subject
     * @param \Magento\Rma\Api\Data\RmaInterface $result
     * @return \Magento\Rma\Api\Data\RmaInterface
     */
    public function afterGet(\Magento\Rma\Api\RmaRepositoryInterface $subject, \Magento\Rma\Api\Data\RmaInterface $result)
    {
        try{

            if(!$result->getItems()) {
                $orderItems = $this->orderRepository->get($result->getOrderId())->getItems();
                $tmpItems = [];
                foreach ($orderItems as $orderItem){
                    $orderItemId = $orderItem->getItemId();
                    $tmpItems = $tmpItems + $this->itemCollectionFactory->create()->addFieldToSelect('*')
                                                ->addFieldToFilter('order_item_id',$orderItemId)
                                                ->addFieldToFilter('rma_entity_id',$result->getEntityId())
                                                ->getItems();
                }
                $rmaItems = $tmpItems;
                $result->setItems($rmaItems);
            }else{
                $rmaItems = $result->getItems();
            }
            foreach ($rmaItems as $rmaItem) {
                $attributesUpdated = false;
                $attributes = $rmaItem->getExtensionAttributes();
                if ($attributes === null) {
                    /** @var \Magento\Rma\Api\Data\ItemExtensionFactory $attributes */
                    $attributes = $this->extensionInterfaceFactory->create();
                }
                if($changeSizeOrderId = $this->rmaExtensionAttributesRepository->getChangeSizeOrderId($rmaItem->getEntityId())){
                   $attributes->setOrderIncrementId($this->orderRepository->get($changeSizeOrderId)->getIncrementId());
                   $attributesUpdated = true;
                }

                if($this->getStandardReasonByReasonId($rmaItem->getReason())!=""){
                    $attributes->setStandardReason($this->getStandardReasonByReasonId($rmaItem->getReason()));
                    $attributesUpdated = true;
                }
                if($attributesUpdated){
                    $rmaItem->setExtensionAttributes($attributes);
                }
            }
            return $result;
        }catch (\Exception $e){
            $this->logger->critical($e->getMessage());
            return $result;
        }catch (\Throwable $t){
            $this->logger->critical($t->getMessage());
            return $result;
        }

    }

    /**
     * @param int $reasonId
     * @return string
     */
    public function getStandardReasonByReasonId(int $reasonId){
        $rmaReasonId = $reasonId;
        if($this->rmaHelper->checkIfItIsMapped($rmaReasonId)){
            $standardReason = $this->rmaHelper->getMappedReasonByReasonId($rmaReasonId,'brand');
            if($standardReason) {
                return $standardReason[0];
            }else{
                return "";
            }
        }else{
            return "";
        }
    }


}